MODULE_NAME='PLENA Comms' (DEV VD_COMMS,	// VIRTUAL DEVICE
			   INTEGER BASE)	// BASE PORT 

(***********************************************************)
(* System Type : NetLinx                                   *)
(***********************************************************)
(*           CENTRAL COMMUNICATION MODULE FOR              *) 
(*       BOSCH PLENA PLM-4Px2x DSP POWER AMPIFIERS         *)
(*        & BOSCH PLENA PLM-8M8 DSP MATRIX MIXERS          *)
(*                                                         *)
(*      	BOSCH COMMUNICATIONS SYSTEMS               *)
(*    		    25 HUNTINGWOOD DRIVE                   *)
(*    		  HUNTINGWOOD NSW AUSTRALIA                *)
(*                                                         *)
(*	            VERSION 1.1 FEB 2014                   *)
(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

(* SYSTEM *)
MAX_DEVICES			= 10
MAX_ENTRIES			= 10
MAX_ENTRY_SIZE			= 128
MAX_TX_DATA_SIZE		= 64
MAX_RX_DATA_SIZE		= 300
QUEUE_SIZE			= 10

(* PLENA PLM CONSTANTS *)
TX_PORT				= 12128		// TRASMISSION UDP PORT
RX_PORT				= 12129		// RECEPTION UDP PORT

MIXER_ID			= $5E40
AMP_ID				= $5E41

MASTER_ID			= $0001
SLAVE_ID			= $0100

(* TIME LINES *)
TL_TICK				= 1	// 100MS TIMER
(***********************************************************)
(*              DATA TYPE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_TYPE

STRUCTURE _DEV
{
    CHAR	ADDR[32]
    INTEGER	NO
    INTEGER 	TYPE
//    INTEGER	STATUS
    DEV		TX
    CHAR	TX_CMD[4]
    CHAR	TX_DATA[MAX_TX_DATA_SIZE]
    INTEGER	TX_ENABLED
}

STRUCTURE _LINK
{
    INTEGER	ENABLED
    CHAR	RX_HEADER[10]
    INTEGER	RX_NO
    CHAR	RX_CMD[4]
    CHAR	RX_DATA[MAX_RX_DATA_SIZE]
    INTEGER     RX_ID
    INTEGER     RX_SEQ_NO
    INTEGER	RX_DATA_SIZE
}

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

_DEV		DEVICE[MAX_DEVICES]
_LINK		LINK
INTEGER		SEQ_NO
LONG 		TL_TICK_COUNT[] 	= { 100 }
DEV 		RX 			= 0:0:0 // RX PORT
DEV		TXS[MAX_DEVICES]

INTEGER		GX
(***********************************************************)
(*               LATCHING DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_LATCHING

(***********************************************************)
(*       MUTUALLY EXCLUSIVE DEFINITIONS GO BELOW           *)
(***********************************************************)
DEFINE_MUTUALLY_EXCLUSIVE

(***********************************************************)
(*        SUBROUTINE/FUNCTION DEFINITIONS GO BELOW         *)
(***********************************************************)
DEFINE_FUNCTION CHAR[255] DISPLAY_STRING (CHAR S[])
{
    CHAR RES[255]
    INTEGER I
 
    RES = ''
    FOR (I = 1; I <= LENGTH_STRING (S); I ++)
    {
	IF (S[I] >= ' ' && S[I] <= '~')
	    RES = "RES, S[I]"
	ELSE
	    RES = "RES, '<', ITOA (S[I]), '>'"
    }
  	RETURN RES
}

DEFINE_FUNCTION CHAR[255] HEX_STRING (S_[])
{
    INTEGER X
    CHAR RES[255]
  
    RES = ''
    FOR (X = 1; X <= LENGTH_STRING (S_); X ++)
	RES = "RES, ' ', FORMAT ('%.2X', S_[X])"
    RETURN RES
}  

DEFINE_FUNCTION TERMINAL (CHAR S[])
{
// NON DEBUG VERSION
}

DEFINE_FUNCTION CHAR[2] WORD (INTEGER W)
{
    RETURN "W / $100, W % $100"
}

DEFINE_FUNCTION SEND_PACKET (INTEGER DEV_)	// COMPOSE PACKET AND SEND 
{
    CHAR TMP[512]

    IF (SEQ_NO = $FFFF)
	SEQ_NO = 1
    ELSE
	SEQ_NO ++
    TMP = "WORD (DEVICE[DEV_].TYPE), WORD (MASTER_ID), WORD (SEQ_NO),
	   WORD (0), WORD (LENGTH_STRING (DEVICE[DEV_].TX_CMD) + LENGTH_STRING (DEVICE[DEV_].TX_DATA)), 
	   DEVICE[DEV_].TX_CMD, DEVICE[DEV_].TX_DATA"
    SEND_STRING DEVICE[DEV_].TX, TMP
    DEVICE[DEV_].TX_CMD = ''
    DEVICE[DEV_].TX_DATA = ''
}

DEFINE_FUNCTION DECODED ()
{
    IF (LINK.RX_CMD = 'SMON' || LINK.RX_CMD = 'EXPL' || LINK.RX_NO = 0) RETURN;
    SEND_STRING VD_COMMS, "LINK.RX_NO, LINK.RX_CMD, LINK.RX_DATA"
}

DEFINE_FUNCTION CONNECT (INTEGER STATE_)
{
    INTEGER I
    
    LINK.ENABLED = STATE_
    IF (STATE_)
	IP_SERVER_OPEN (RX.PORT, RX_PORT, 2)
    ELSE
    {
	FOR (I = 1; I <= MAX_DEVICES; I ++)
	{
	    DEVICE[I].TX_CMD = ''
	    DEVICE[I].TX_DATA = ''
	    IF (DEVICE[I].TX_ENABLED)
		IP_CLIENT_CLOSE (DEVICE[I].TX.PORT)
	    DEVICE[I].TX_ENABLED = 0
	}
	IP_SERVER_CLOSE (RX.PORT)
    }
}

DEFINE_FUNCTION INIT ()
{
    INTEGER I

    FOR (I = 1; I <= MAX_DEVICES; I ++)
    {
	DEVICE[I].ADDR = ''
	DEVICE[I].NO = I
	DEVICE[I].TYPE = 0
	DEVICE[I].TX.NUMBER = 0
	DEVICE[I].TX.PORT = BASE + I + 1
	DEVICE[I].TX.SYSTEM = 0
	DEVICE[I].TX_ENABLED = 0
    }
    LINK.ENABLED = 0
    LINK.RX_HEADER = ''
    LINK.RX_DATA = ''
    LINK.RX_CMD = ''
    LINK.RX_ID = 0
    LINK.RX_SEQ_NO = 0
    LINK.RX_DATA_SIZE = 0
    SEQ_NO = 0
}

(***********************************************************)
(*                STARTUP CODE GOES BELOW                  *)
(***********************************************************)
DEFINE_START

INIT ()
RX.PORT = BASE
SET_LENGTH_ARRAY (TXS, 0)
REBUILD_EVENT ()
CONNECT (1)

TIMELINE_CREATE (TL_TICK, TL_TICK_COUNT, 1, TIMELINE_ABSOLUTE, TIMELINE_REPEAT)

(***********************************************************)
(*                THE EVENTS GO BELOW                      *)
(***********************************************************)
DEFINE_EVENT

(* SOCKET TRAFFIC *)
DATA_EVENT [TXS]
{
    ONLINE  : 
    {
	INTEGER I
	
	I = DATA.DEVICE.PORT - (BASE + 1)
	TERMINAL ("'TXS ', ITOA (I), ' ONLINE '")
	DEVICE[I].TX_ENABLED = 1
	SEND_PACKET (I)
    }
    OFFLINE  : 
    {
	INTEGER I
	
	I = DATA.DEVICE.PORT - (BASE + 1)
	TERMINAL ("'TXS ', ITOA (I), ' OFFLINE '")
	DEVICE[I].TX_ENABLED = 0
    }
}

DATA_EVENT [RX]
{
    STRING  :
    {
	INTEGER X
	
	LINK.RX_NO = 0
	FOR (X = 1; X <= MAX_DEVICES; X ++) 
	    IF (DEVICE[X].ADDR = DATA.SOURCEIP)
	    {
		LINK.RX_NO = X
		BREAK
	    }
	WHILE (LENGTH_STRING (DATA.TEXT) >= 14)
	{
	    LINK.RX_HEADER = GET_BUFFER_STRING (DATA.TEXT, 10)
	    LINK.RX_SEQ_NO = $100 * LINK.RX_HEADER[5] + LINK.RX_HEADER[6]
	    LINK.RX_DATA_SIZE = $100 * LINK.RX_HEADER[9] + LINK.RX_HEADER[10] - 4
	    LINK.RX_CMD = GET_BUFFER_STRING (DATA.TEXT, 4)
	    IF (LINK.RX_DATA_SIZE = 0)
	        LINK.RX_DATA = ''
	    ELSE
		LINK.RX_DATA = GET_BUFFER_STRING (DATA.TEXT, LINK.RX_DATA_SIZE)
	    DECODED ()
	}
    }
}

(* VIRTUAL DEVICE *)
BUTTON_EVENT [VD_COMMS, 0]
{
    PUSH :
    {
	SWITCH (BUTTON.INPUT.CHANNEL)
	{
	    CASE  1 : CONNECT (1)
	    CASE  2 : CONNECT (0)
	    CASE 11 : ;
	}
    }
}

DATA_EVENT [VD_COMMS]
{
    COMMAND :
    {
	INTEGER NO_, X, FOUND
	CHAR CMD_[4]
	
	IF (LENGTH_STRING (DATA.TEXT) >= 5)
	{
	    NO_ = GET_BUFFER_CHAR (DATA.TEXT)
	    CMD_ = GET_BUFFER_STRING (DATA.TEXT, 4)
//	    TERMINAL ("'NO ', ITOA (NO_), ' CMD ', CMD_, ' DATA ', ITOA (LENGTH_STRING (DATA.TEXT))")
	    IF (CMD_[1] = '@')	// SPECIAL COMMANDS
	    {
		IF (CMD_ = '@AMP' || CMD_ = '@MIX')	// REGISTER AMP OR MIXER DEVICE
		{
		    DEVICE[NO_].ADDR = DATA.TEXT
		    IF (CMD_ = '@AMP') DEVICE[NO_].TYPE = AMP_ID
		    ELSE DEVICE[NO_].TYPE = MIXER_ID
		    TERMINAL ("'DEVICE ', ITOA (NO_), ' SET TO ', DEVICE[NO_].ADDR")
		    IF (DEVICE[NO_].TX_ENABLED)
		    {
			IP_CLIENT_CLOSE (DEVICE[NO_].TX.PORT)
			DEVICE[NO_].TX_ENABLED = 0
		    }
		    FOUND = 0
		    FOR (X = 1; X <= LENGTH_ARRAY (TXS); X ++)
			IF (TXS[X].PORT = DEVICE[NO_].TX.PORT) 
			{
			    FOUND = 1
			    BREAK
			}
		    IF (!FOUND)
		    {
			X = LENGTH_ARRAY (TXS)
			SET_LENGTH_ARRAY (TXS, X + 1)
			TXS[X + 1] = DEVICE[NO_].TX
			REBUILD_EVENT ()
		    }
		}
	    }
	    ELSE
	    {
		DEVICE[NO_].TX_CMD = CMD_
		DEVICE[NO_].TX_DATA = DATA.TEXT
		IF (DEVICE[NO_].TX_ENABLED)
		    SEND_PACKET (NO_)
		ELSE
		    IP_CLIENT_OPEN (DEVICE[NO_].TX.PORT, DEVICE[NO_].ADDR, TX_PORT, 2)
	    }
	}
    }
}

(***********************************************************)
(*            THE ACTUAL PROGRAM GOES BELOW                *)
(***********************************************************)
DEFINE_PROGRAM


(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)

