PROGRAM_NAME='PLENA PLM-8M8 Demo'
(***********************************************************)
(* System Type : NetLinx                                   *)
(***********************************************************)
(*  		    DEMO PROGRAM FOR                       *) 
(*         BOSCH PLENA PLM-8M8 DSP MATRIX MIXERS           *)
(*                                                         *)
(*      	BOSCH COMMUNICATIONS SYSTEMS               *)
(*    		    25 HUNTINGWOOD DRIVE                   *)
(*    		  HUNTINGWOOD NSW AUSTRALIA                *)
(*                                                         *)
(*	            VERSION 1.0 FEB 2014                   *)
(***********************************************************)
(*    DEVELOPMENT SYSTEM :-			           *)
(*      NI-3000 	F/W v2.32.148		           *)
(*	NXD-500I 	F/W v2.4.5			   *)
(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE
dvTPA				= 10001:1:1	// TOUCH PANEL - GENERAL
dvTPB				= 10001:2:1	// TOUCH PANEL - ZONE 1 / COMMON
dvTPC				= 10001:3:1	// TOUCH PANEL - ZONE 2
dvTPD				= 10001:4:1	// TOUCH PANEL - ZONE 3
dvTPE				= 10001:5:1	// TOUCH PANEL - ZONE 4
dvTPF				= 10001:6:1	// TOUCH PANEL - ZONE 5
dvTPG				= 10001:7:1	// TOUCH PANEL - ZONE 6
dvTPH				= 10001:8:1	// TOUCH PANEL - ZONE 7
dvTPI				= 10001:9:1	// TOUCH PANEL - ZONE 8

vdCOMMS				= 33001:1:0	// COMMS DEVICE
vdMIXER1			= 33002:1:0	// MIXER

IP_TERMINAL			= 8005:1:1
(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

INTEGER		MAX_ENTRIES	= 10	
INTEGER    	MAX_ENTRY_SIZE	= 128

DEV		TP_ZONES[]	= {dvTPB, dvTPC, dvTPD, dvTPE,
				   dvTPF, dvTPG, dvTPH, dvTPI} 

(***********************************************************)
(*              DATA TYPE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_TYPE

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

CHAR		MIXER1_ADDR[]	= '192.168.0.31'

INTEGER		BASE_PORT 	= 10
INTEGER		MIXER1_ID	= 1

INTEGER		CURRENT_FADER[8]			// CURRENT FADER BEING PRESSED
CHAR		PAGE_NAME[20] 	= 'Title'		// CURRENT PAGE NAME
CHAR		TERMINAL_ONLINE = 0

CHAR		ENTRY[MAX_ENTRIES][MAX_ENTRY_SIZE]	// DECODED DATA ENTRIES
INTEGER		NO_ENTRIES				// NUMBER OF ENTRIES

INTEGER		MIXER_LEVEL, MIXER_VALUE

CHAR		TAPER_TABLE[256], 
		ANTI_TAPER_TABLE[256]
(***********************************************************)
(*               LATCHING DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_LATCHING

(***********************************************************)
(*       MUTUALLY EXCLUSIVE DEFINITIONS GO BELOW           *)
(***********************************************************)
DEFINE_MUTUALLY_EXCLUSIVE

(***********************************************************)
(*        SUBROUTINE/FUNCTION DEFINITIONS GO BELOW         *)
(***********************************************************)
DEFINE_FUNCTION TERMINAL (MSG_[])
{
    IF (TERMINAL_ONLINE)
	SEND_STRING IP_TERMINAL, MSG_
}

DEFINE_FUNCTION SHOW_DETAILS ()
{
    TERMINAL ("'Program Details...'")
    TERMINAL ("'PROGRAM FILE : ', __FILE__, ' DATED ', __DATE__")
    TERMINAL ("'PROGRAM NAME : ', __NAME__")
    TERMINAL ("'COMPILER VERSION : ', __VERSION__")
}     

DEFINE_FUNCTION CHAR[255] HEX_STRING (S_[])
{
    INTEGER X
    CHAR RES[255]
  
    RES = ''
    FOR (X = 1; X <= LENGTH_STRING (S_); X ++)
	RES = "RES, ' ', FORMAT ('%.2X', S_[X])"
    RETURN RES
}  

DEFINE_FUNCTION CHAR[255] DISPLAY_LINE (CHAR S_[])  
{
    INTEGER X
    CHAR RES[255]
  
    RES = ''
    FOR (X = 1; X <= LENGTH_STRING (S_); X ++)
	IF (S_[X] >= ' ' && S_[X] <= '~')
	    RES = "RES, S_[X]"
	ELSE  
	    RES = "RES, '<', ITOA (S_[X]), '>'"
    RETURN RES
}

DEFINE_FUNCTION CHAR[12] LVL_TO_DB (INTEGER LEVEL_)
{
    CHAR TMP[12]
    SINTEGER X

    IF (LEVEL_ >= 249)
	RETURN '24dB'
    ELSE IF (LEVEL_ <= 1)
	RETURN '-100.0dB'
    ELSE
    {
	X = (TYPE_CAST (LEVEL_) - 201) * 5
	RETURN "ITOA (X / 10), '.', ITOA (ABS_VALUE (X % 10)), 'dB'"
    }
}

DEFINE_FUNCTION SHOW_FEEDBACK ()
{
    [dvTPA, 11] = (PAGE_NAME = 'Zone 1')
    [dvTPA, 12] = (PAGE_NAME = 'Zone 2')
    [dvTPA, 13] = (PAGE_NAME = 'Zone 3')
    [dvTPA, 14] = (PAGE_NAME = 'Zone 4')
    [dvTPA, 15] = (PAGE_NAME = 'Zone 5')
    [dvTPA, 16] = (PAGE_NAME = 'Zone 6')
    [dvTPA, 17] = (PAGE_NAME = 'Zone 7')
    [dvTPA, 18] = (PAGE_NAME = 'Zone 8')
    [dvTPA, 19] = (PAGE_NAME = 'Presets')    
}

DEFINE_FUNCTION SHOW_PAGE (CHAR PAGE_[])
{
    PAGE_NAME = PAGE_
    SEND_COMMAND dvTPA, "'PAGE-', PAGE_"
    SHOW_FEEDBACK ()
}

DEFINE_FUNCTION DECODE_DATA (INTEGER NO_, CHAR DATA_[])
{
    INTEGER I, X, Y, Z
	
    // THIS DEMO IS FOR ONE MIXER ONLY. 
    // IF MORE ARE USED, THEN THE "NO_" PARAMATER WILL NEED TO BE TAKEN INTO ACCOUNT
    NO_ENTRIES = 0
    X = FIND_STRING (DATA_, ':', 1)
    WHILE (X)
    {
	NO_ENTRIES ++
	ENTRY[NO_ENTRIES] = LEFT_STRING (DATA_, X - 1)
	GET_BUFFER_STRING (DATA_, X)
	X = FIND_STRING (DATA_, ':', 1)
    }
    NO_ENTRIES ++
    ENTRY[NO_ENTRIES] = DATA.TEXT
    FOR (X = 1; X <= NO_ENTRIES; X ++)
	TERMINAL ("'  ', ITOA (X), '  "', ENTRY[X], '"'")
    IF (NO_ENTRIES = 1)
    {
	SWITCH (ENTRY[1])
	{
	    CASE 'ONLINE'  : TERMINAL ("'MIXER ', ITOA (NO_), ' ONLINE'")
	    CASE 'OFFLINE' : TERMINAL ("'MIXER ', ITOA (NO_), ' OFFLINE'")
	}
    }
    ELSE IF (NO_ENTRIES = 2)
    {
	SWITCH (ENTRY[1])
	{
	    CASE 'NAME'   : SEND_COMMAND TP_ZONES[1], "'^TXT-9,1&2,', ENTRY[2]"
	    CASE 'POWER'  : [TP_ZONES[1], 2] = (ENTRY[2] = '1')
	    CASE 'MUTE'   : [TP_ZONES[1], 4] = (ENTRY[2] = '1')
	    CASE 'ONLINE' : [TP_ZONES[1], 8] = (ENTRY[2] = '1')
	    CASE 'STATUS' : 
	    {
		[TP_ZONES[1], 51] = (ENTRY[2] = '0')
		[TP_ZONES[1], 52] = (ENTRY[2] = '1')
		[TP_ZONES[1], 53] = (ENTRY[2] = '2')
		[TP_ZONES[1], 54] = (ENTRY[2] = '3')
		[TP_ZONES[1], 55] = (ENTRY[2] = '4')
		[TP_ZONES[1], 2] = (ENTRY[2] <> '1')	// POWER
	    }
	    CASE 'ERR'    : TERMINAL ("'MIXER ', ITOA (NO_), ' ERROR $', ENTRY[2]") 
	}
    }
    ELSE IF (NO_ENTRIES = 3)
    {
	X = ATOI (ENTRY[2])
	SWITCH (ENTRY[1])
	{
	    CASE 'MIC_NAME'  : SEND_COMMAND TP_ZONES[1], "'^TXT-', ITOA (X), ',1&2,', ENTRY[3]"
	    CASE 'BGM_NAME'  : SEND_COMMAND TP_ZONES[1], "'^TXT-', ITOA (X + 4), ',1&2,', ENTRY[3]"
	    CASE 'ZONE_NAME' : SEND_COMMAND TP_ZONES[1], "'^TXT-', ITOA (X + 8), ',1&2,', ENTRY[3]"
	}
    }
    ELSE IF (NO_ENTRIES = 4)
    {
	SWITCH (ENTRY[1])
	{
	    CASE 'PAGING' :
	    {	// 2 ZONE No, 3 LEVEL, 4 MUTE
		X = ATOI (ENTRY[2])	// ZONE No
		Y = ATOI (ENTRY[3])	// LEVEL
		TERMINAL ("'PAGING ZONE', ITOA (X), '  ', LVL_TO_DB (Y), ' MUTE ', ENTRY[4]")
		[TP_ZONES[X], 16] = (ENTRY[4] = '1')	// MUTE
		IF (CURRENT_FADER[X] <> 6) SEND_LEVEL TP_ZONES[X], 6, ANTI_TAPER_TABLE[Y + 1]
		SEND_COMMAND TP_ZONES[X], "'^TXT-26,1&2,', LVL_TO_DB (Y)"
	    }
	    CASE 'MASTER' :
	    {	// 2 ZONE No, 3 LEVEL, 4 MUTE
		X = ATOI (ENTRY[2])	// ZONE No
		Y = ATOI (ENTRY[3])	// LEVEL
		TERMINAL ("'MASTER ZONE', ITOA (X), '  ', LVL_TO_DB (Y), ' MUTE ', ENTRY[4]")
		[TP_ZONES[X], 17] = (ENTRY[4] = '1')
		IF (CURRENT_FADER[X] <> 7) SEND_LEVEL TP_ZONES[X], 7, ANTI_TAPER_TABLE[Y + 1]
		SEND_COMMAND TP_ZONES[X], "'^TXT-27,1&2,', LVL_TO_DB (Y)"
	    }
	    CASE 'PRESET' : 
	    {	// 2 PRESET No, 3 VALID
		X = ATOI (ENTRY[2])
		Y = ATOI (ENTRY[3])
		SEND_COMMAND TP_ZONES[1], "'^TXT-', ITOA (X + 30), ',1&2,', ENTRY[4]"
		[TP_ZONES[1], X + 70] = (Y)	
	    }
	}
    }
    ELSE IF (NO_ENTRIES = 5)
    {
	SWITCH (ENTRY[1])
	{
	    CASE 'BGM' :
	    {	// 2 ZONE No, 3 LEVEL, 4 MUTE, 5 SELECTION
		X = ATOI (ENTRY[2]) 		// ZONE No
		Y = ATOI (ENTRY[3])		// LEVEL
		TERMINAL ("'BGM ZONE', ITOA (X), '  ', LVL_TO_DB (Y), ' MUTE ', ENTRY[5]")
		[TP_ZONES[X], 15] = (ENTRY[4] = '1')
		[TP_ZONES[X], 21] = (ENTRY[5] = '1') // CHECK
		[TP_ZONES[X], 22] = (ENTRY[5] = '2')
		[TP_ZONES[X], 23] = (ENTRY[5] = '3')
		[TP_ZONES[X], 24] = (ENTRY[5] = '0')
		IF (CURRENT_FADER[X] <> 5) SEND_LEVEL TP_ZONES[X], 5, ANTI_TAPER_TABLE[Y + 1]
		SEND_COMMAND TP_ZONES[X], "'^TXT-25,1&2,', LVL_TO_DB (Y)"
	    }
	    CASE 'MIC' :
	    {	// 2 MIC No, 3 ZONE No, 4 LEVEL, 5 MUTE
		X = ATOI (ENTRY[3])		// ZONE No 	
		Y = ATOI (ENTRY[4])		// LEVEL
		Z = ATOI (ENTRY[2])		// MIC No
		TERMINAL ("'MIC', ITOA (Z), ' ZONE', ITOA (X), ' ', LVL_TO_DB (Y), ' MUTE ', ENTRY[5]")
		[TP_ZONES[X], 10 + Z] = (ENTRY[5] = '1')
		IF (CURRENT_FADER[X] <> Z) SEND_LEVEL TP_ZONES[X], Z, ANTI_TAPER_TABLE[Y + 1]
		SEND_COMMAND TP_ZONES[X], "'^TXT-', ITOA (20 + Z), ',1&2,', LVL_TO_DB (Y)"
	    }
	}
    }
}

DEFINE_FUNCTION INIT ()
{
    INTEGER I
    
    FOR (I = 1; I <= MAX_ENTRIES; I ++)
	ENTRY[I] = ''
    NO_ENTRIES = 0
    FOR (I = 1; I <= 8; I ++)
	CURRENT_FADER[I] = 0
   
    TAPER_TABLE =	// THESE TABLES CONVERT THE FADERS FROM LIN-LOG TO LOG-LOG TAPER  
	"$00, $03, $05, $08, $0B, $0D, $10, $12, $15, $18, $1A, $1D, $1F, $22, $25, $27,
	 $2A, $2C, $2F, $31, $34, $36, $39, $3B, $3E, $40, $43, $45, $48, $4A, $4C, $4F,
	 $51, $53, $56, $58, $5A, $5C, $5F, $61, $63, $65, $67, $69, $6B, $6D, $6F, $71,
	 $73, $75, $77, $79, $7B, $7C, $7E, $80, $81, $83, $85, $86, $88, $89, $8A, $8C,
	 $8D, $8E, $8F, $91, $92, $93, $94, $95, $96, $97, $97, $98, $99, $9A, $9B, $9B,
	 $9C, $9D, $9D, $9E, $9E, $9F, $A0, $A0, $A1, $A1, $A2, $A2, $A3, $A3, $A4, $A4,
	 $A5, $A6, $A6, $A7, $A7, $A8, $A8, $A9, $A9, $AA, $AA, $AB, $AB, $AC, $AC, $AD,
	 $AD, $AE, $AE, $AF, $AF, $B0, $B0, $B1, $B1, $B2, $B2, $B3, $B3, $B4, $B4, $B5,
	 $B5, $B5, $B6, $B6, $B7, $B7, $B7, $B8, $B8, $B8, $B9, $B9, $B9, $BA, $BA, $BA,
	 $BA, $BB, $BB, $BB, $BC, $BC, $BC, $BC, $BD, $BD, $BD, $BE, $BE, $BE, $BE, $BF,
	 $BF, $BF, $C0, $C0, $C0, $C0, $C1, $C1, $C1, $C2, $C2, $C2, $C3, $C3, $C3, $C4,
	 $C4, $C4, $C5, $C5, $C5, $C5, $C6, $C6, $C6, $C7, $C7, $C7, $C8, $C8, $C8, $C9,
	 $C9, $C9, $CA, $CA, $CA, $CB, $CB, $CB, $CC, $CC, $CC, $CC, $CD, $CD, $CD, $CE,
	 $CE, $CE, $CF, $CF, $CF, $D0, $D0, $D0, $D1, $D1, $D1, $D1, $D2, $D2, $D2, $D3,
	 $D3, $D3, $D4, $D4, $D4, $D5, $D5, $D5, $D5, $D6, $D6, $D6, $D7, $D7, $D7, $D8,
	 $D8, $D8, $D9, $D9, $D9, $DA, $DA, $DA, $DA, $DB, $DB, $DB, $DC, $DC, $DC, $DD" 

    ANTI_TAPER_TABLE = 
	"$00, $01, $01, $01, $02, $02, $03, $03, $03, $04, $04, $04, $05, $05, $06, $06,
	 $06, $07, $07, $08, $08, $08, $09, $09, $09, $0A, $0A, $0B, $0B, $0B, $0C, $0C,
	 $0D, $0D, $0D, $0E, $0E, $0E, $0F, $0F, $10, $10, $10, $11, $11, $12, $12, $12,
	 $13, $13, $14, $14, $14, $15, $15, $16, $16, $16, $17, $17, $18, $18, $18, $19,
	 $19, $1A, $1A, $1A, $1B, $1B, $1C, $1C, $1C, $1D, $1D, $1E, $1E, $1F, $1F, $1F,
	 $20, $20, $21, $21, $22, $22, $22, $23, $23, $24, $24, $25, $25, $26, $26, $26,
	 $27, $27, $28, $28, $29, $29, $2A, $2A, $2B, $2B, $2C, $2C, $2D, $2D, $2E, $2E,
	 $2F, $2F, $30, $30, $31, $31, $32, $32, $33, $33, $34, $34, $35, $36, $36, $37,
	 $37, $38, $39, $39, $3A, $3A, $3B, $3C, $3C, $3D, $3E, $3F, $3F, $40, $41, $42,
	 $43, $43, $44, $45, $46, $47, $48, $49, $4B, $4C, $4D, $4E, $50, $51, $53, $55,
	 $56, $58, $5A, $5C, $5E, $60, $61, $63, $65, $67, $69, $6B, $6D, $6F, $71, $73,
	 $75, $77, $79, $7B, $7D, $7F, $82, $84, $87, $8A, $8D, $91, $94, $98, $9B, $9F,
	 $A2, $A6, $A9, $AC, $AF, $B2, $B6, $B9, $BC, $BF, $C2, $C5, $C8, $CC, $CF, $D2,
	 $D5, $D8, $DC, $DF, $E2, $E5, $E9, $EC, $EF, $F2, $F5, $F9, $FC, $FF, $FF, $FF,
	 $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF,
	 $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF" 
}

(***********************************************************)
(*                STARTUP CODE GOES BELOW                  *)
(***********************************************************)
DEFINE_START
INIT ()
SET_VIRTUAL_LEVEL_COUNT (dvTPB, 20)	// FIX FOR NXD-500i

DEFINE_MODULE 'PLENA Comms' COMMS1 (vdCOMMS, BASE_PORT)
DEFINE_MODULE 'PLENA PLM-8M8' PLM1 (vdMIXER1, vdCOMMS, MIXER1_ADDR, MIXER1_ID)

(***********************************************************)
(*                THE EVENTS GO BELOW                      *)
(***********************************************************)
DEFINE_EVENT

(* TERMINAL *)     
DATA_EVENT [IP_TERMINAL]
{
    ONLINE  : 
    { 
	TERMINAL_ONLINE = 1 
	SHOW_DETAILS () 
    }
    OFFLINE : TERMINAL_ONLINE = 0
}

BUTTON_EVENT [IP_TERMINAL ,0]
{
    PUSH    : 
    {
	SWITCH (BUTTON.INPUT.CHANNEL)
	{
	    CASE  1 : 
	    CASE  2 : 
	    CASE  3 : 
	    CASE  4 : DO_PUSH_TIMED (vdMIXER1, BUTTON.INPUT.CHANNEL, DO_PUSH_TIMED_INFINITE)
	
	    CASE  5 : 
	    CASE  6 : 
	    CASE  7 : 
	    
	    CASE 11 : DO_PUSH_TIMED (vdMIXER1, 31, DO_PUSH_TIMED_INFINITE)
	}
    }
    RELEASE : 
    {
	SWITCH (BUTTON.INPUT.CHANNEL)
	{
	    CASE  1 : 
	    CASE  2 :
	    CASE  3 :
	    CASE  4 : DO_RELEASE (vdMIXER1, BUTTON.INPUT.CHANNEL)
	    
	    CASE  5 : 
	    CASE  6 :
	    CASE  7 :
	
	    CASE 11 : DO_RELEASE (vdMIXER1, 31)
	}
    }
}     

DATA_EVENT [dvTPA]
{
    ONLINE  : SHOW_PAGE ('Title')
}

DATA_EVENT [dvTPB]
{
    ONLINE : SEND_COMMAND vdMIXER1, 'REFRESH'
}

BUTTON_EVENT [dvTPA, 0]
{
    PUSH    :
    {
	TERMINAL ("'TPA PUSH ', ITOA (BUTTON.INPUT.CHANNEL)")
	SWITCH (BUTTON.INPUT.CHANNEL)
	{
	    CASE 11 :
	    CASE 12 :
	    CASE 13 :
	    CASE 14 : 
	    CASE 15 :
	    CASE 16 : 
	    CASE 17 :
	    CASE 18 : SHOW_PAGE ("'Zone ', ITOA (BUTTON.INPUT.CHANNEL - 10)")
	    CASE 19 : SHOW_PAGE ('Presets')
	}
	SHOW_FEEDBACK ()
    }
    RELEASE :
    {
    }
}

BUTTON_EVENT [TP_ZONES, 0] 
{
    PUSH    : 
    {
	INTEGER ZONE, BTN
	
	BTN = BUTTON.INPUT.CHANNEL
	ZONE = GET_LAST (TP_ZONES)
	TERMINAL ("'TP Z', ITOA (ZONE), ' PUSH ', ITOA (BTN)")
	SWITCH (BTN)
	{
	    CASE   2 : SEND_COMMAND vdMIXER1, 'POWER:T'
	    CASE   4 : SEND_COMMAND vdMIXER1, 'MUTE:T'
	
	    CASE  11 : SEND_COMMAND vdMIXER1, "'MIC_MUTE:1:', ITOA (ZONE), ':T'"
	    CASE  12 : SEND_COMMAND vdMIXER1, "'MIC_MUTE:2:', ITOA (ZONE), ':T'"
	    CASE  13 : SEND_COMMAND vdMIXER1, "'MIC_MUTE:3:', ITOA (ZONE), ':T'"
	    CASE  14 : SEND_COMMAND vdMIXER1, "'MIC_MUTE:4:', ITOA (ZONE), ':T'"
	    CASE  15 : SEND_COMMAND vdMIXER1, "'BGM_MUTE:', ITOA (ZONE), ':T'"
	    CASE  16 : SEND_COMMAND vdMIXER1, "'PAGING_MUTE:', ITOA (ZONE), ':T'"
	    CASE  17 : SEND_COMMAND vdMIXER1, "'MASTER_MUTE:', ITOA (ZONE), ':T'"
	    
	    CASE  21 : SEND_COMMAND vdMIXER1, "'BGM_SELECT:', ITOA (ZONE), ':1'"
	    CASE  22 : SEND_COMMAND vdMIXER1, "'BGM_SELECT:', ITOA (ZONE), ':2'"
	    CASE  23 : SEND_COMMAND vdMIXER1, "'BGM_SELECT:', ITOA (ZONE), ':3'"
	    CASE  24 : SEND_COMMAND vdMIXER1, "'BGM_SELECT:', ITOA (ZONE), ':0'"
	 
	    CASE  81 : SEND_COMMAND vdMIXER1, 'PRESET:1'
	    CASE  82 : SEND_COMMAND vdMIXER1, 'PRESET:2'
	    CASE  83 : SEND_COMMAND vdMIXER1, 'PRESET:3'
	    CASE  84 : SEND_COMMAND vdMIXER1, 'PRESET:4'
	    CASE  85 : SEND_COMMAND vdMIXER1, 'PRESET:5'
	    
	    DEFAULT :
	    {
		IF (BTN > 40 && BTN <= 47)
		    CURRENT_FADER[ZONE] = BTN - 40
	    }
	}
    }
    RELEASE : 
    {
	INTEGER ZONE, BTN
	
	BTN = BUTTON.INPUT.CHANNEL
	ZONE = GET_LAST (TP_ZONES)
	TERMINAL ("'TP Z', ITOA (ZONE), ' RELEASE ', ITOA (BTN)")
	SWITCH (BTN)
	{
	    CASE  1 : ;
	    DEFAULT :
	    {
		IF (BTN > 40 && BTN <= 47)
		    CURRENT_FADER[ZONE] = 0
	    }
	}
    }
}

LEVEL_EVENT [TP_ZONES, 0]
{
    INTEGER ZONE, X, Y
    
    ZONE = GET_LAST (TP_ZONES)
    IF (CURRENT_FADER[ZONE] = LEVEL.INPUT.LEVEL) // LEVEL BEING CHANGED ON TP
    {
	TERMINAL ("'TP Z', ITOA (ZONE), '  ', ITOA (LEVEL.INPUT.LEVEL), ' = ', ITOA (LEVEL.VALUE), ' (', ITOA (TAPER_TABLE[LEVEL.VALUE + 1]), ')'")
	SWITCH (LEVEL.INPUT.LEVEL)
	{
	    CASE 1 :  // MIC 1 - 4
	    CASE 2 :
	    CASE 3 :
	    CASE 4 :
		SEND_COMMAND vdMIXER1, "'MIC_LEVEL:', ITOA (LEVEL.INPUT.LEVEL), ':', ITOA (ZONE), ':', ITOA (TAPER_TABLE[LEVEL.VALUE + 1])"
	    CASE 5 :	// BGM
		SEND_COMMAND vdMIXER1, "'BGM_LEVEL:', ITOA (ZONE), ':', ITOA (TAPER_TABLE[LEVEL.VALUE + 1])"
	    CASE 6 :	// PAGING
		SEND_COMMAND vdMIXER1, "'PAGING_LEVEL:', ITOA (ZONE), ':', ITOA (TAPER_TABLE[LEVEL.VALUE + 1])"
	    CASE 7 : 	// MASTER
		SEND_COMMAND vdMIXER1, "'MASTER_LEVEL:', ITOA (ZONE), ':', ITOA (TAPER_TABLE[LEVEL.VALUE + 1])"
	}
    }
}

// DECODE STRING RESPONSES FROM MIXER MODULES
DATA_EVENT [vdMIXER1]
{
    STRING : DECODE_DATA (1, DATA.TEXT)
}

(***********************************************************)
(*            THE ACTUAL PROGRAM GOES BELOW                *)
(***********************************************************)
DEFINE_PROGRAM

(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)

